
/* A basic 6 channel transmitter using the nRF24L01 module.
 * This code is used for RC-Hoovercraft.
 * Original code by ELECTRONUBES.com
 * http://www.youtube/c/electronoobs 
 *
 *Install the NRF24 library to your IDE
 *Import the servo library as well
 *Upload this code to the Arduino Nano
 *Connect a NRF24 module to it
 *Last working revision 2-2-22
 *
 *Module // Arduino NANO

    Connections from 24L01 Tranceiver module to NANO :
    GND    ->   GND
    Vcc    ->   3.3V
    CE     ->   D10
    CSN    ->   D9
    CLK    ->   D13
    MOSI   ->   D11
    MISO   ->   D12

This code transmits 6 channels and create a PWM output for each one on D2, D3, D4, D5, D6 and D7
 *
*/

 
#include <SPI.h>
#include <nRF24L01.h>
#include <RF24.h>

/*Create a unique pipe out. The receiver has to 
  wear the same unique code*/

/*Create a unique Address to link Transmitter and Receiver and setup a data channel for communication called "pipe"
 * The Receiver needs to have the same address!!
*/
const uint64_t pipeOut = 0xE8E8F0F0E1LL;    // This address needs to be the same in the receiver: 0xE8E8F0F0E1LL
RF24 radio(10, 9); // select CE / CSN  pin  //select CE / CSN  pin for nRF24L01 tranceiver module


// The size of this struct should not exceed 32 bytes
// setting up struct to send the data package to Receiver
struct MyData {
  byte throttle;                            //PWM output for Servo  Throttle for propellor
  byte yaw;                                 //PWM output for Servo  not used for hovercraft
  byte pitch;                               //PWM output for Servo  not used for hovercraft
  byte roll;                                //PWM output for Servo  Rudder for steering
  byte AUX1;                                // Auxilary output 1  not used for hovercraft
  byte AUX2;                                // Auxilary output 2  not used for hovercraft
};

MyData data;                                //With the name "data" you you call the parts you need from the struct 

void resetData()                            // reset the struct values
{
  //We define the initial value of each data input
  //All 4 potentiometers will be in the middle position so 127 is the middle from 254
  data.throttle = 127;                      // 127 sets the 0 output in the center op the Joystick.
  data.yaw = 127;                           // same here
  data.pitch = 127;                         // same here
  data.roll = 127;                          // same here
  data.AUX1 = 0;                            // Aux output set to low
  data.AUX2 = 0;                            // Aux output set to low
}

/******************Setting up all values that need to be setup********************************/

void setup()
{
  Serial.begin(9600);                  // set communication speed
  
  //Configure the nRF24L01 tranceiver module
  radio.begin();                      // Start Tranceiver 
  radio.setAutoAck(false);            // Set Autoacknoledgement to OFF
  radio.setDataRate(RF24_250KBPS);    // set data speed to 250KBytes per second
  radio.openWritingPipe(pipeOut);     // Open Channel (pipe) for transmitting data
  
  
  resetData();                        // reset variables
}

/**************Configuration routine for setting up the Joystick position************************************/

// Returns a corrected value for a joystick position that takes into account
// the values of the outer extents and the middle of the joystick range.
int mapJoystickValues(int val, int lower, int middle, int upper, bool reverse)
{
  val = constrain(val, lower, upper);
  if ( val < middle )
    val = map(val, lower, middle, 0, 128);
  else
    val = map(val, middle, upper, 128, 255);
  return ( reverse ? 255 - val : val );
}

/****************Program Loop**********************************/
void loop()
{
  // The calibration numbers used here should be measured 
  // for your joysticks till they send the correct values.
  // for each joystick the mapJoystickValues routine is called to set the boundries 
  data.throttle = mapJoystickValues( analogRead(A0), 13, 524, 1015, true ); // read analog joystick on port A0 and store data in struct
  data.yaw      = mapJoystickValues( analogRead(A1),  1, 505, 1020, true ); // read analog joystick on port A1 and store data in struct
  data.pitch    = mapJoystickValues( analogRead(A2), 12, 544, 1021, true ); // read analog joystick on port A2 and store data in struct
  data.roll     = mapJoystickValues( analogRead(A3), 34, 522, 1020, true ); // read analog joystick on port A3 and store data in struct
  
  data.AUX1     = digitalRead(7);       // in this code but not used for hoovercraft
  data.AUX2     = digitalRead(8);       // in this code but not used for hoovercraft
  
  radio.write(&data, sizeof(MyData));   // send all data stored in Struct "MyData" and send this data package to the tranceiver
}
 
